@@ -35,10 +35,15 @@ del _cloud_tpu_init
 from jax import config as _config_module
 del _config_module
 
+# Force early import, allowing use of `jax.core` after importing `jax`.
+import jax.core as _core
+del _core
+
 # Note: import <name> as <name> is required for names to be exported.
 # See PEP 484 & https://github.com/google/jax/issues/7570
 
 from jax._src.basearray import Array as Array
+from jax import typing as typing
 
 from jax._src.config import (
   config as config,
@@ -66,7 +71,6 @@ from jax._src.core import ensure_compile_time_eval as ensure_compile_time_eval
 from jax._src.environment_info import print_environment_info as print_environment_info
 from jax._src.api import (
   Device as Device,
-  ad,  # TODO(phawkins): update users to avoid this.
   effects_barrier,
   block_until_ready as block_until_ready,
   checkpoint as checkpoint,
@@ -110,17 +114,19 @@ from jax._src.api import (
   process_count as process_count,
   process_index as process_index,
   pure_callback as pure_callback,
-  pxla,  # TODO(phawkins): update users to avoid this.
   remat as remat,
   ShapedArray,  # TODO(jakevdp): update users to avoid this.
   ShapeDtypeStruct as ShapeDtypeStruct,
   value_and_grad as value_and_grad,
   vjp as vjp,
   vmap as vmap,
-  xla,  # TODO(phawkins): update users to avoid this.
   xla_computation as xla_computation,
 )
 
+from jax.interpreters import ad # TODO(phawkins): update users to avoid this.
+from jax.interpreters import pxla # TODO(phawkins): update users to avoid this.
+from jax.interpreters import xla # TODO(phawkins): update users to avoid this.
+
 from jax._src.array import (
     make_array_from_single_device_arrays as make_array_from_single_device_arrays,
     make_array_from_callback as make_array_from_callback,
