@@ -308,18 +308,6 @@ class ArrayImpl(basearray.Array):
         # here after uneven partitioning support is added.
         return (api.device_put(self._value[i]) for i in range(self.shape[0]))
 
-  def item(self):
-    if dtypes.issubdtype(self.dtype, np.complexfloating):
-      return complex(self)
-    elif dtypes.issubdtype(self.dtype, np.floating):
-      return float(self)
-    elif dtypes.issubdtype(self.dtype, np.integer):
-      return int(self)
-    elif dtypes.issubdtype(self.dtype, np.bool_):
-      return bool(self)
-    else:
-      raise TypeError(self.dtype)
-
   @property
   def is_fully_replicated(self) -> bool:
     return self.shape == self._arrays[0].shape
@@ -595,8 +583,8 @@ def make_array_from_single_device_arrays(
 
   Example:
 
-    >>> from jax.experimental.maps import Mesh
-    >>> from jax.experimental import PartitionSpec as P
+    >>> from jax.sharding import Mesh
+    >>> from jax.sharding import PartitionSpec as P
     >>> import numpy as np
     ...
     >>> shape = (8, 8)
