@@ -469,7 +469,9 @@ class Config:
     return (axis_env_state, self.x64_enabled, self.jax_numpy_rank_promotion,
             self.jax_default_matmul_precision, self.jax_dynamic_shapes,
             self.jax_numpy_dtype_promotion, self.jax_default_device,
-            self.jax_array, self.jax_threefry_partitionable)
+            self.jax_array, self.jax_threefry_partitionable,
+            # Technically this affects jaxpr->MHLO lowering, not tracing.
+            self.jax_hlo_source_file_canonicalization_regex)
 
 class NoDefault: pass
 no_default = NoDefault()
@@ -780,9 +782,13 @@ jax_array = config.define_bool_state(
 
 jit_pjit_api_merge = config.define_bool_state(
     name='jax_jit_pjit_api_merge',
-    default=False,
+    default=True,
     upgrade=True,
-    help=('If True, jit and pjit API will be merged.'))
+    help=('If True, jit and pjit API will be merged. You can only disable it via '
+          "the environment variable i.e. `os.environ['JAX_JIT_PJIT_API_MERGE'] = '0'`. "
+          "The merge must be disabled via an environment variable since it "
+          "affects JAX at import time so it needs to be disabled before jax is "
+          "imported."))
 
 
 spmd_mode = config.define_enum_state(
@@ -1045,11 +1051,6 @@ config.define_bool_state(
     )
 )
 
-config.define_bool_state(
-    name='jax_experimental_subjaxpr_lowering_cache',
-    default=False,
-    help='Enable using a cache for lowering subjaxprs.')
-
 # TODO(sharadmv,mattjj): set default to True, then remove
 config.define_bool_state(
     name='jax_eager_pmap',
