@@ -23,10 +23,7 @@ from jax.tree_util import (tree_flatten, tree_unflatten, tree_map,
                            register_pytree_node_class, tree_leaves)
 from jax.errors import UnexpectedTracerError
 from jax.interpreters import partial_eval as pe
-from jax.interpreters import batching
-from jax.interpreters import mlir
 from jax.interpreters import xla
-from jax.interpreters.batching import not_mapped
 from jax.config import config
 
 from jax._src import core
@@ -38,6 +35,9 @@ from jax._src.ad_util import Zero, zeros_like_aval, stop_gradient_p
 from jax._src.api_util import argnums_partial, flatten_fun_nokwargs
 from jax._src.core import raise_to_shaped
 from jax._src.interpreters import ad
+from jax._src.interpreters import batching
+from jax._src.interpreters import mlir
+from jax._src.interpreters.batching import not_mapped
 from jax._src.lax import lax
 from jax._src.util import cache, safe_zip, safe_map, split_list, Unhashable
 
@@ -764,7 +764,8 @@ def _custom_vjp_call_jaxpr_vmap(spmd_axis_name,
   in_batched = [d is not not_mapped for d in in_dims]
   _, args_batched = split_list(in_batched, [num_consts])
   batched_fun_jaxpr, out_batched = batching.batch_jaxpr(
-      fun_jaxpr, axis_size, in_batched, False, axis_name, main_type)
+      fun_jaxpr, axis_size, in_batched, False, axis_name, spmd_axis_name,
+      main_type)
   out_dims1 = [0 if b else not_mapped for b in out_batched]
   out_dims2 = []
 
@@ -772,7 +773,8 @@ def _custom_vjp_call_jaxpr_vmap(spmd_axis_name,
   def batched_fwd_jaxpr_thunk():
     fwd_jaxpr = core.ClosedJaxpr(*fwd_jaxpr_thunk())  # consts can be tracers
     batched_fwd_jaxpr, out_batched = batching.batch_jaxpr(
-        fwd_jaxpr, axis_size, args_batched, False, axis_name, main_type)
+        fwd_jaxpr, axis_size, args_batched, False, axis_name, spmd_axis_name,
+        main_type)
     out_dims2.append([0 if b else not_mapped for b in out_batched])
     return batched_fwd_jaxpr.jaxpr, batched_fwd_jaxpr.consts
 
