@@ -15,8 +15,6 @@
 import functools
 from typing import Any, Callable, Optional, Tuple
 
-from jax._src import linear_util as lu
-from jax.interpreters import ad
 from jax.interpreters import mlir
 from jax.interpreters import partial_eval as pe
 from jax.interpreters import xla
@@ -26,9 +24,11 @@ from jax._src import ad_util
 from jax._src import api_util
 from jax._src import core
 from jax._src import custom_api_util
+from jax._src import linear_util as lu
 from jax._src import source_info_util
 from jax._src import traceback_util
 from jax._src import util
+from jax._src.interpreters import ad
 
 
 source_info_util.register_exclusion(__file__)
