@@ -27,8 +27,6 @@ from jax import tree_util
 from jax import lax
 from jax.config import config
 from jax.experimental import pjit
-from jax.interpreters import batching
-from jax.interpreters import mlir
 from jax.interpreters import partial_eval as pe
 from jax.interpreters import pxla
 
@@ -38,11 +36,13 @@ from jax._src import custom_derivatives
 from jax._src import linear_util as lu
 from jax._src import util
 from jax._src.interpreters import ad
+from jax._src.interpreters import batching
+from jax._src.interpreters import mlir
 from jax._src.lax import control_flow as lcf
 from jax._src.lib import xla_client as xc
 from jax._src.lib.mlir import ir
 from jax._src.lib.mlir.dialects import hlo
-from jax._src.sharding import Sharding, OpShardingSharding
+from jax._src.sharding import Sharding, OpShardingSharding, NamedSharding
 
 # pytype: disable=import-error
 try:
@@ -311,7 +311,7 @@ def _inspect_sharding_lowering_rule(ctx: mlir.LoweringRuleContext, value, *,
         devices, op_sharding))
     pspec = pjit.parse_flatten_op_sharding(
         op_sharding, mesh)[0].get_partition_spec()
-    return callback(pjit.NamedSharding(mesh, pspec))
+    return callback(NamedSharding(mesh, pspec))
 
   if len(devices) == 1:
     # If we only have one device in our computation, we can construct a trivial
@@ -562,8 +562,8 @@ def inspect_array_sharding(value, *, callback: Callable[[Sharding], None]):
 
   >>> import jax
   >>> import jax.numpy as jnp
-  >>> from jax.experimental.maps import Mesh
-  >>> from jax.experimental.pjit import PartitionSpec, pjit
+  >>> from jax.experimental.pjit import pjit
+  >>> from jax.sharding import Mesh, PartitionSpec
   >>>
   >>> x = jnp.arange(8, dtype=jnp.float32)
   >>> def f_(x):
