@@ -22,7 +22,7 @@ import itertools
 import time
 from typing import (
     Any, Callable, Dict, Iterable, Iterator, Optional, Protocol,
-    Sequence, Set, Tuple, List, Type, Union)
+    Sequence, Set, Tuple, List, Type, Union, NamedTuple)
 import logging
 import os
 import re
@@ -34,9 +34,7 @@ import numpy as np
 import jax
 from jax.errors import UnexpectedTracerError
 from jax.monitoring import record_event_duration_secs
-import jax.interpreters.batching as batching
 import jax.interpreters.mlir as mlir
-import jax._src.interpreters.xla as xla
 from jax.interpreters import pxla
 import jax.interpreters.partial_eval as pe
 
@@ -47,12 +45,15 @@ from jax._src import dtypes
 from jax._src import linear_util as lu
 from jax._src import path
 from jax._src import profiler
+from jax._src import source_info_util
 from jax._src import stages
 from jax._src import traceback_util
 from jax._src import util
 from jax._src.abstract_arrays import array_types
 from jax._src.config import config, flags
 from jax._src.interpreters import ad
+from jax._src.interpreters import batching
+from jax._src.interpreters import xla
 from jax._src.lib.mlir import ir
 from jax._src.lib.mlir.dialects import use_stablehlo
 from jax._src.lib import pmap_lib
@@ -102,7 +103,7 @@ _on_exit = False
 ArgSpec = Tuple[core.AbstractValue, Optional[Device]]
 
 def arg_spec(x: Any) -> ArgSpec:
-  from jax.experimental import pjit
+  from jax._src import pjit
 
   aval = xla.abstractify(x)
   try:
@@ -553,20 +554,33 @@ def jaxpr_has_primitive(jaxpr, prim_name: str):
   return False
 
 
-def jaxpr_shardings(jaxpr) -> Iterator[jax.sharding.XLACompatibleSharding]:
-  from jax.experimental import pjit, shard_map
+class SourceInfo(NamedTuple):
+  source_info: str
+  eqn_name: str
+
+
+def jaxpr_shardings(
+    jaxpr) -> Iterator[Tuple[jax.sharding.XLACompatibleSharding, SourceInfo]]:
+  from jax._src import pjit
+  from jax.experimental import shard_map
 
   for eqn in jaxpr.eqns:
     if eqn.primitive is pjit.sharding_constraint_p:
-      yield eqn.params['sharding']
+      source_info = SourceInfo(source_info_util.summarize(eqn.source_info),
+                                eqn.primitive.name)
+      yield (eqn.params['sharding'], source_info)
     elif eqn.primitive is pjit.pjit_p:
-      yield from eqn.params['in_shardings']
-      yield from eqn.params['out_shardings']
+      source_info = SourceInfo(source_info_util.summarize(eqn.source_info),
+                                eqn.primitive.name)
+      yield from ((i, source_info) for i in eqn.params['in_shardings'])
+      yield from ((o, source_info) for o in eqn.params['out_shardings'])
     elif eqn.primitive is shard_map.shard_map_p:
+      source_info = SourceInfo(source_info_util.summarize(eqn.source_info),
+                                eqn.primitive.name)
       def _names_to_pspec(names):
         ndmin = max(names) + 1 if names else 0
         return PartitionSpec(*(names.get(i) for i in range(ndmin)))
-      yield from (NamedSharding(eqn.params['mesh'], _names_to_pspec(names))
+      yield from ((NamedSharding(eqn.params['mesh'], _names_to_pspec(names)), source_info)
                   for names in eqn.params['in_names'])
   for subjaxpr in core.subjaxprs(jaxpr):
     yield from jaxpr_shardings(subjaxpr)
