@@ -24,7 +24,6 @@ import warnings
 import numpy as np
 
 from jax import tree_util
-from jax.interpreters import batching
 from jax.interpreters import mlir
 from jax.interpreters import pxla
 from jax.interpreters import xla
@@ -34,6 +33,7 @@ from jax._src import dtypes
 from jax._src import util
 from jax._src.core import ShapedArray, AxisName, raise_to_shaped
 from jax._src.interpreters import ad
+from jax._src.interpreters import batching
 from jax._src.lax import lax
 from jax._src.lax import slicing
 from jax._src.lib.mlir import ir
