@@ -20,15 +20,18 @@ import warnings
 
 import numpy as np
 
-from jax import core
 from jax import lax
 from jax._src import api
+from jax._src import core
 from jax._src import dtypes
 from jax._src.numpy.ndarray import ndarray
-from jax._src.numpy.util import _broadcast_to, _check_arraylike, _complex_elem_type, _promote_dtypes_inexact, _promote_dtypes_numeric, _where, _wraps
+from jax._src.numpy.util import (
+    _broadcast_to, _check_arraylike, _complex_elem_type,
+    _promote_dtypes_inexact, _promote_dtypes_numeric, _where, _wraps)
 from jax._src.lax import lax as lax_internal
 from jax._src.typing import Array, ArrayLike, DType, DTypeLike
-from jax._src.util import canonicalize_axis as _canonicalize_axis, maybe_named_axis, prod as _prod
+from jax._src.util import (
+    canonicalize_axis as _canonicalize_axis, maybe_named_axis, prod as _prod)
 
 
 _all = builtins.all
@@ -72,7 +75,7 @@ def _reduction(a: ArrayLike, name: str, np_fun: Any, op: ReductionOp, init_val:
                axis: Axis = None, dtype: DTypeLike = None, out: None = None,
                keepdims: bool = False, initial: Optional[ArrayLike] = None,
                where_: Optional[ArrayLike] = None,
-               parallel_reduce: Optional[Callable[..., ArrayLike]] = None,
+               parallel_reduce: Optional[Callable[..., Array]] = None,
                promote_integers: bool = False) -> Array:
   bool_op = bool_op or op
   # Note: we must accept out=None as an argument, because numpy reductions delegate to
@@ -131,7 +134,11 @@ def _reduction(a: ArrayLike, name: str, np_fun: Any, op: ReductionOp, init_val:
   else:
     result = lax.reduce(a, init_val, op, dims)
   if initial is not None:
-    result = op(lax.convert_element_type(initial, _asarray(a).dtype), result)
+    initial_arr = lax.convert_element_type(initial, _asarray(a).dtype)
+    if initial_arr.shape != ():
+      raise ValueError("initial value must be a scalar. "
+                       f"Got array of shape {initial_arr.shape}")
+    result = op(initial_arr, result)
   if keepdims:
     result = lax.expand_dims(result, pos_dims)
   return lax.convert_element_type(result, dtype or result_dtype)
