@@ -24,7 +24,6 @@ import jax
 import jax.numpy as jnp
 from jax import lax
 from jax.config import config
-from jax.interpreters import batching
 from jax.interpreters import mlir
 from jax.numpy.linalg import cholesky, svd, eigh
 
@@ -34,6 +33,7 @@ from jax._src import prng
 from jax._src.api import jit, vmap
 from jax._src.core import NamedShape
 from jax._src.interpreters import ad
+from jax._src.interpreters import batching
 from jax._src.lax import lax as lax_internal
 from jax._src.lib import xla_bridge
 from jax._src.numpy.lax_numpy import (
