@@ -21,6 +21,7 @@ from typing import (Sequence, List, Tuple, Optional, Mapping, Dict, Set,
                     FrozenSet, Union, cast)
 
 import jax
+from jax._src import core
 from jax._src.util import safe_map, safe_zip, use_cpp_class, use_cpp_method
 from jax._src.lib import xla_client as xc
 from jax.interpreters import mlir
@@ -483,7 +484,7 @@ class PmapSharding(XLACompatibleSharding):
     """
     # The dtype doesn't matter here. Its only used for creating the
     # sharding_spec.
-    aval = jax.core.ShapedArray(shape, np.int32)
+    aval = core.ShapedArray(shape, np.int32)
     sharding_spec = pxla._create_pmap_sharding_spec(aval, sharded_dim)
 
     num_ways_sharded = None
