@@ -20,14 +20,14 @@ from typing import Any, Dict, List, Optional, Protocol, Sequence, Tuple, Union
 
 import numpy as np
 
-from jax import core
 from jax import lax
-from jax._src import linear_util as lu
 from jax.interpreters import partial_eval as pe
-from jax._src.util import safe_map, safe_zip, split_list
 
+from jax._src import core
+from jax._src import linear_util as lu
 from jax._src.state.types import ShapedArrayRef
 from jax._src.state.primitives import get_p, swap_p, addupdate_p
+from jax._src.util import safe_map, safe_zip, split_list
 
 ## JAX utilities
 
