@@ -16,19 +16,21 @@ from functools import partial
 
 from typing import Any, List, Protocol, Tuple, TypeVar, Union
 
-from jax import core
+import numpy as np
+
 from jax import lax
-from jax._src import ad_util
-from jax._src import pretty_printer as pp
-from jax._src.typing import Array
-from jax._src.util import safe_map, safe_zip, partition_list, tuple_insert
 from jax.interpreters import ad
 from jax.interpreters import batching
 from jax.interpreters import partial_eval as pe
-import numpy as np
 
+from jax._src import ad_util
+from jax._src import core
+from jax._src import pretty_printer as pp
+from jax._src.typing import Array
 from jax._src.state.types import (ShapedArrayRef, ReadEffect, WriteEffect,
                                   AccumEffect)
+from jax._src.util import safe_map, safe_zip, partition_list, tuple_insert
+
 
 ## General utilities
 
@@ -227,19 +229,18 @@ def _pp_idx(context, non_slice_idx, indexed_dims):
   assert next(idx_iter, None) is None
   return pp.text(idx)
 
-def _get_pp_rule(eqn, context, settings):
+def _get_pp_rule(eqn, context, settings) -> pp.Doc:
   # Pretty prints `a = get x i` as `x[i] <- a`
   y, = eqn.outvars
   x, *idx = eqn.invars
   idx = _pp_idx(context, idx, eqn.params["indexed_dims"])
   lhs = core.pp_vars([y], context, print_shapes=settings.print_shapes)
   # TODO more general get
-  return [lhs, pp.text(' <- '), pp_ref(pp.concat([
-    pp.text(core.pp_var(x, context)), pp.text('['), idx, pp.text(']')
-    ]))]
+  return pp.concat([lhs, pp.text(' <- '), pp_ref(pp.concat([
+      pp.text(core.pp_var(x, context)), pp.text('['), idx, pp.text(']')]))])
 core.pp_eqn_rules[get_p] = _get_pp_rule
 
-def _swap_pp_rule(eqn, context, settings):
+def _swap_pp_rule(eqn, context, settings) -> pp.Doc:
   y, = eqn.outvars
   x, v, *idx = eqn.invars
   idx = _pp_idx(context, idx, eqn.params["indexed_dims"])
@@ -247,30 +248,31 @@ def _swap_pp_rule(eqn, context, settings):
     # In the case of a set (ignored return value),
     # pretty print `_ = swap x v i` as `x[i] <- v`
     del y
-    return [
-      pp_ref(pp.concat([
-        pp.text(core.pp_var(x, context)),
-        pp.text('['), idx, pp.text(']')
-      ])), pp.text(' <- '), pp.text(core.pp_var(v, context))]
+    return pp.concat([
+        pp_ref(pp.concat([
+            pp.text(core.pp_var(x, context)),
+            pp.text('['), idx, pp.text(']')
+        ])), pp.text(' <- '), pp.text(core.pp_var(v, context))])
   else:
     # pretty-print `y:T = swap x v i` as `y:T, x[i] <- x[i], v`
     x_i = pp.concat([pp.text(core.pp_var(x, context)),
                      pp.text('['), idx, pp.text(']')])
     y = core.pp_vars([y], context, print_shapes=settings.print_shapes)
-    return [y, pp.text(', '), pp_ref(x_i), pp.text(' <- '),
-            pp_ref(x_i), pp.text(', '), pp.text(core.pp_var(v, context))]
+    return pp.concat([y, pp.text(', '), pp_ref(x_i), pp.text(' <- '),
+                      pp_ref(x_i), pp.text(', '),
+                      pp.text(core.pp_var(v, context))])
 core.pp_eqn_rules[swap_p] = _swap_pp_rule
 
-def _addupdate_pp_rule(eqn, context, settings):
+def _addupdate_pp_rule(eqn, context, settings) -> pp.Doc:
   # pretty-print ` = addupdate x i v` as `x[i] += v`
   () = eqn.outvars
   x, v, *idx = eqn.invars
   idx = _pp_idx(context, idx, eqn.params["indexed_dims"])
-  return [
+  return pp.concat([
     pp_ref(pp.concat([
         pp.text(core.pp_var(x, context)),
         pp.text('['), idx, pp.text(']')
-      ])), pp.text(' += '), pp.text(core.pp_var(v, context))]
+    ])), pp.text(' += '), pp.text(core.pp_var(v, context))])
 core.pp_eqn_rules[addupdate_p] = _addupdate_pp_rule
 
 ## get/swap/addupdate JVP rules
