@@ -35,7 +35,7 @@ import numpy.random as npr
 import jax
 from jax import lax
 from jax.experimental.compilation_cache import compilation_cache
-from jax.interpreters import mlir
+from jax._src.interpreters import mlir
 from jax.tree_util import tree_map, tree_all, tree_flatten, tree_unflatten
 from jax._src import api
 from jax._src import pjit as pjit_lib
@@ -1014,6 +1014,8 @@ _CPP_JIT_IMPLEMENTATION = functools.partial(api._jit, True)
 _CPP_JIT_IMPLEMENTATION._name = "cpp"
 _PYTHON_JIT_IMPLEMENTATION = functools.partial(api._jit, False)
 _PYTHON_JIT_IMPLEMENTATION._name = "python"
+_PJIT_IMPLEMENTATION = jax.jit
+_PJIT_IMPLEMENTATION._name = "pjit"
 _NOOP_JIT_IMPLEMENTATION = lambda x, *args, **kwargs: x
 _NOOP_JIT_IMPLEMENTATION._name = "noop"
 
