@@ -27,7 +27,6 @@ from jax._src.core import (
   AxisPrimitive as AxisPrimitive,
   AxisSize as AxisSize,
   AxisSubst as AxisSubst,
-  Bot as Bot,
   CallPrimitive as CallPrimitive,
   ClosedCallPrimitive as ClosedCallPrimitive,
   ClosedJaxpr as ClosedJaxpr,
@@ -115,12 +114,7 @@ from jax._src.core import (
   as_named_shape as as_named_shape,
   attrgetter as attrgetter,
   aval_mapping_handlers as aval_mapping_handlers,
-  aval_method as aval_method,
-  aval_property as aval_property,
   axis_frame as axis_frame,
-  axis_substitution_rules as axis_substitution_rules,
-  bint as bint,
-  bot as bot,
   call as call,
   call_bind_with_continuation as call_bind_with_continuation,
   call_impl as call_impl,
