@@ -14,7 +14,7 @@
 
 import jax
 import inspect
-from jax import core
+from jax._src import core
 from jax import tree_util
 from jax._src import linear_util as lu
 from jax.experimental import pjit
@@ -22,7 +22,7 @@ from jax.errors import UnexpectedTracerError
 from jax._src.lib.mlir.dialects import hlo
 from jax._src.lib.mlir import ir
 import jax.interpreters.pxla as pxla
-from jax.interpreters import mlir
+from jax._src.interpreters import mlir
 from jax.interpreters import partial_eval as pe
 from jax._src import custom_api_util
 from jax._src.lib import xla_client as xc
@@ -60,7 +60,7 @@ _CUSTOM_PARTITIONING_CALL_NAME = "CustomSPMDPartitioning"
 
 
 def _to_jax_shape(s):
-  return jax.core.ShapedArray(s.dimensions(), s.numpy_dtype())
+  return core.ShapedArray(s.dimensions(), s.numpy_dtype())
 
 
 def _custom_partitioning_propagate_user_sharding(sharding, shape, backend_string):
@@ -213,7 +213,7 @@ class custom_partitioning:
       from jax.experimental.custom_partitioning import custom_partitioning
       from jax.experimental.pjit import pjit
       from jax.sharding import PartitionSpec as P
-      from jax.experimental.maps import Mesh
+      from jax.sharding import Mesh
       from jax.numpy.fft import fft
       import regex as re
       import numpy as np
@@ -389,7 +389,7 @@ def _custom_partitioning_lowering_rule(ctx: mlir.LoweringRuleContext, *values,
       return OpShardingSharding(devices, op_sharding)
     pspec = pjit.parse_flatten_op_sharding(op_sharding,
                                            mesh)[0].get_partition_spec()
-    return pjit.NamedSharding(mesh, pspec)
+    return jax.sharding.NamedSharding(mesh, pspec)
 
   sharding_callback_info = _ShardingCallbackInfo(propagate_user_sharding, partition,
                                                 to_mesh_pspec_sharding,
