@@ -19,7 +19,7 @@ import numpy as np
 from typing import Callable, Sequence, Tuple, Union, Mapping, Optional, List, Dict, NamedTuple
 
 import jax
-from jax import core
+from jax._src import core
 from jax._src import dispatch
 from jax._src import api_util
 from jax._src.lib import xla_bridge as xb
@@ -538,8 +538,8 @@ class GlobalDeviceArray:
 
     Example:
 
-      >>> from jax.experimental.maps import Mesh
-      >>> from jax.experimental import PartitionSpec as P
+      >>> from jax.sharding import Mesh
+      >>> from jax.sharding import PartitionSpec as P
       >>> import numpy as np
       ...
       >>> global_input_shape = (8, 2)
@@ -585,8 +585,8 @@ class GlobalDeviceArray:
 
     Example:
 
-      >>> from jax.experimental.maps import Mesh
-      >>> from jax.experimental import PartitionSpec as P
+      >>> from jax.sharding import Mesh
+      >>> from jax.sharding import PartitionSpec as P
       >>> import numpy as np
       ...
       >>> global_input_shape = (8, 2)
