@@ -29,9 +29,9 @@ from typing import Any, Callable, Optional, Sequence, Tuple
 from absl import logging
 
 import jax
-from jax import core
 from jax import dlpack
 from jax import dtypes
+from jax import numpy as jnp
 from jax import tree_util
 from jax._src import util
 from jax._src import ad_util
@@ -40,6 +40,7 @@ from jax._src import ad_checkpoint
 from jax._src import custom_derivatives
 from jax.interpreters import mlir
 from jax.interpreters import xla
+from jax._src import core
 from jax._src.lib.mlir import ir
 from jax._src.lib.mlir.dialects import func as func_dialect
 from jax._src.lib.mlir.dialects import hlo
@@ -116,14 +117,27 @@ def call_tf(callable_tf: Callable, has_side_effects=True) -> Callable:
       return tf.TensorSpec(a_tf_shape, a_tf_dtype)
     args_flat_sig_tf = tuple(map(make_tensorspec, args_flat_jax))
 
+    def check_tf_result(r_tf):
+      # Check that the TF function returns values of expected types. This
+      # improves error reporting, preventing hard-to-diagnose errors downstream
+      try:
+        jax2tf_internal._tfval_to_tensor_jax_dtype(r_tf)
+      except Exception as e:
+        msg = ("The called TF function returns a result that is not "
+               f"convertible to JAX: {r_tf}.")
+        raise ValueError(msg) from e
+
     res_treedef = None  # We'll store here the result treedef
+    res_tf_flat = None  # For error reporting
     # The function below will be called at least once, either in eager
     # or in graph mode.
     def callable_flat_tf(*args_tf_flat: TfVal) -> Sequence[TfVal]:
       args_tf = args_treedef.unflatten(args_tf_flat)
       res_tf = callable_tf(*args_tf)
-      nonlocal res_treedef
+      nonlocal res_treedef, res_tf_flat
       res_tf_flat, res_treedef_now = tree_util.tree_flatten(res_tf)
+      for r_tf in res_tf_flat:
+        check_tf_result(r_tf)
       assert res_treedef is None or res_treedef == res_treedef_now, f"Subsequent calls had different results. Previous {res_treedef} and now {res_treedef_now}"
       res_treedef = res_treedef_now
       return res_tf_flat
@@ -158,6 +172,21 @@ def call_tf(callable_tf: Callable, has_side_effects=True) -> Callable:
         function_flat_tf=function_flat_tf,
         args_flat_sig_tf=args_flat_sig_tf,
         has_side_effects=has_side_effects)
+
+    assert res_treedef is not None
+    # Sometimes, in compiled mode, we get a different number of results than we
+    # got when tracing the TF function (and building the res_treedef). This
+    # can happen, e.g., when returning tf.TensorArray, which appears as one
+    # leaf when tracing but after compilation we get a tuple. See
+    # call_tf_test.test_error_bad_result_tensorarray.
+    if res_treedef.num_leaves != len(res_jax_flat):
+      # It is not clear if this error can happen once we have check_tf_result
+      # in callable_flat_tf, but we keep it for safety.
+      msg = (f"Incorrect number of results ({len(res_jax_flat)}) from the "
+             "called TF function after compilation. "
+             f"Expected {res_treedef.num_leaves} leaves based on observed "
+             f"results during tracing: {res_tf_flat}.")
+      raise ValueError(msg)
     return res_treedef.unflatten(res_jax_flat)
 
   # Define the fwd and bwd custom_vjp functions
@@ -172,9 +201,9 @@ def call_tf(callable_tf: Callable, has_side_effects=True) -> Callable:
       """Invoke TF gradient."""
 
       # TF does not like us to watch non-float vars
-      def replace_non_float(arg):
-        if np.issubdtype(arg.dtype.as_numpy_dtype, np.inexact):
-          return arg
+      def replace_non_float(arg_tf):
+        if arg_tf.dtype.is_floating or arg_tf.dtype.is_complex:
+          return arg_tf
         else:
           # When watched, this will be ignored. When use in results it will
           # result in a floating 0. gradient, which JAX will ignore (and
@@ -221,8 +250,8 @@ call_tf_p.multiple_results = True
 def _call_tf_impl(*args_jax_flat, callable_flat_tf, **_):
   # On GPU we use dlpack to avoid copies of data to the host.
   def _arg_jax_to_tf(arg_jax):
-    if (isinstance(arg_jax, xla.DeviceArray) and
-        arg_jax.device_buffer.client.platform in _DLPACK_PLATFORMS and
+    if (isinstance(arg_jax, jax.Array) and
+        arg_jax.device().platform in _DLPACK_PLATFORMS and
         arg_jax.dtype in dlpack.SUPPORTED_DTYPES):
       arg_dlpack = jax.dlpack.to_dlpack(arg_jax, take_ownership=False)
       return tf.experimental.dlpack.from_dlpack(arg_dlpack)
@@ -245,7 +274,12 @@ def _call_tf_impl(*args_jax_flat, callable_flat_tf, **_):
         res_dlpack = tf.experimental.dlpack.to_dlpack(res_tf)
         return jax.dlpack.from_dlpack(res_dlpack)
 
-    return jax.device_put(np.asarray(res_tf))
+    # When working with a bfloat16 scalar tf.Tensor,np.asarray() can fail.
+    # To handle this special case, we create a numpy copy.
+    if res_tf.shape == tf.TensorShape([]) and res_tf.dtype == tf.bfloat16:
+      return jax.device_put(jnp.array(res_tf.numpy()))
+    else:
+      return jax.device_put(np.asarray(res_tf))
 
   return list(map(_res_tf_to_jax, res_tf_flat))
 
@@ -375,28 +409,6 @@ def _code_generator_and_avals(
       raise ValueError(msg) from e
 
   xla_comp = xla_client.XlaComputation(func_tf_hlo)
-  # Check that the function does not have compile-time constant inputs that
-  # have been inlined in the compiled code.
-  xla_comp_parameter_shapes = xla_comp.program_shape().parameter_shapes()
-  found_parameter_avals = [
-      core.ShapedArray(found_xla_shape.dimensions(),
-                       dtypes.canonicalize_dtype(found_xla_shape.numpy_dtype()))
-      for found_xla_shape in xla_comp_parameter_shapes
-  ]
-  # Add the captured_inputs to args_flat_sig_tf
-  expected_args_flat_sig_tf = list(args_flat_sig_tf) + list(captured_inputs)
-  expected_parameter_avals = [
-      core.ShapedArray(tuple(arg_sig.shape.as_list()),
-                       dtypes.canonicalize_dtype(arg_sig.dtype.as_numpy_dtype))
-      for arg_sig in expected_args_flat_sig_tf]
-  if found_parameter_avals != expected_parameter_avals:
-    msg = ("Compiled TensorFlow function has unexpected parameter types " +
-           f"{found_parameter_avals}, while the expected types are " +
-           f"{expected_parameter_avals}. Perhaps the TensorFlow function " +
-           "has shape-influencing inputs, and thus needs to be recompiled " +
-           "for each value of some inputs. " +
-           "See https://github.com/google/jax/blob/main/jax/experimental/jax2tf/README.md#limitations-of-call_tf for a discussion.")
-    raise ValueError(msg)
 
   # Canonicalize the results; e.g., makes them x32 if JAX is in 32-bit mode
   def canonical_res_aval(res_shape: xla.XlaShape) -> core.ShapedArray:
