@@ -18,7 +18,7 @@ from functools import partial, wraps
 import string
 from typing import Any, Callable, Dict, Optional, Sequence, Tuple
 
-from jax import core
+from jax._src import core
 from jax import lax
 from jax._src.lax import slicing as lax_slicing
 from jax._src import dtypes
@@ -1131,3 +1131,5 @@ tf_impl_no_xla[lax.scatter_min_p] = convert_scatter_jax_to_tf(tf.minimum,  tf.ma
 tf_impl_no_xla[lax.scatter_max_p] = convert_scatter_jax_to_tf(tf.maximum,  tf.math.unsorted_segment_max)
 
 tf_impl_no_xla[lax.sort_p] = _unimplemented("sort")
+
+tf_impl_no_xla[lax.reduce_precision_p] = _unimplemented("reduce_precision")
