@@ -117,6 +117,13 @@ class CallTfTest(tf_test_util.JaxToTfTestCase):
     self.assertAllClose(x, res)
     self.assertTrue(np.shares_memory(x, res))
 
+    x = jnp.array(3.0, dtype=jnp.bfloat16)
+    res = jax2tf.call_tf(lambda x: x)(x)
+    self.assertAllClose(x, res)
+    # bfloat16 scalar will create a copy.
+    with self.assertRaises(AssertionError):
+      self.assertTrue(np.shares_memory(x, res))
+
   @_parameterized_jit
   def test_eval_pytree(self, with_jit=True):
 
@@ -129,7 +136,17 @@ class CallTfTest(tf_test_util.JaxToTfTestCase):
     res = fun_jax(x, y)
     self.assertAllClose((np.float32(12.), np.float64(11.)), res)
 
-  def test_eval_non_compileable_strings(self):
+  def test_result_tuple(self):
+    x1 = np.ones(3, dtype=np.int32)
+    x2 = np.ones(5, dtype=np.float32)
+    def fun_tf():
+      return tf.tuple([x1, x2])
+
+    fun_jax = jax.jit(jax2tf.call_tf(fun_tf))
+    res = fun_jax()
+    self.assertAllClose(res, (x1, x2))
+
+  def test_error_non_compileable_strings(self):
     # Check that in op-by-op we call a function in eager mode.
     def f_tf_non_compileable(x):
       return tf.strings.length(tf.strings.format("Hello {}!", [x]))
@@ -145,19 +162,43 @@ class CallTfTest(tf_test_util.JaxToTfTestCase):
                                 _call_tf_non_compileable_error):
       lax.cond(True, lambda x: f_jax(x), lambda x: f_jax(x), x)
 
-  def test_eval_non_compileable_dynamic_shape(self):
+  def test_error_non_compileable_dynamic_shape(self):
     # Check that in op-by-op we call a function in eager mode.
     def f_tf_non_compileable(x):
       return tf.cond(x[0], lambda: x[1:], lambda: x)
 
     f_jax = jax2tf.call_tf(f_tf_non_compileable)
     x = np.array([True, False], dtype=np.bool_)
-    self.assertAllClose(f_tf_non_compileable(x), f_jax(x))
+    self.assertAllClose(f_tf_non_compileable(x), f_jax(x))  # Works in eager mode
 
     with self.assertRaisesRegex(ValueError,
                                 _call_tf_dynamic_shape_error):
       jax.jit(f_jax)(x)
 
+  def test_error_bad_result_tensorarray(self):
+    # Call a function that returns a tf.TensorArray. This should be detected
+    # early on. If we don't the function is actually compileable but returns
+    # a tuple instead of a single result.
+    def fun_tf():
+      ta = tf.TensorArray(tf.int32, size=0, dynamic_size=True)
+      ta = ta.unstack([0, 1, 2, 3, 4])
+      return ta
+
+    with self.assertRaisesRegex(ValueError,
+                                "The called TF function returns a result that is not convertible to JAX"):
+      fun_jax = jax.jit(jax2tf.call_tf(fun_tf))
+      fun_jax()
+
+  def test_error_bad_result_string(self):
+    def fun_tf():
+      return tf.constant("foo")
+
+    # Now under jit, should fail because the function is not compileable
+    with self.assertRaisesRegex(ValueError,
+                                "The called TF function returns a result that is not convertible to JAX"):
+      fun_jax = jax.jit(jax2tf.call_tf(fun_tf))
+      fun_jax()
+
   @_parameterized_jit
   def test_control_flow(self, with_jit=True):
 
@@ -762,6 +803,7 @@ class RoundTripToJaxTest(tf_test_util.JaxToTfTestCase):
     self.assertEqual(1, tracing_count)
 
   def test_custom_grad_saved_model(self):
+
     @jax.custom_vjp
     def f(x):
       return x * x
@@ -778,8 +820,9 @@ class RoundTripToJaxTest(tf_test_util.JaxToTfTestCase):
       return jnp.sum(f(x))
 
     g_tf, _ = tf_test_util.SaveAndLoadFunction(
-        jax2tf.convert(g, with_gradient=True, polymorphic_shapes=["b, ..."]),
-        input_signature=[tf.TensorSpec([None], dtype=tf.float32)])
+        jax2tf.convert(g, with_gradient=True),
+        input_signature=[tf.TensorSpec(shape=(1,), dtype=tf.float32)],
+    )
     g_rt = jax2tf.call_tf(g_tf)
     x = np.array([0.7], dtype=np.float32)
     self.assertAllClose(g(x), g_rt(x))
@@ -832,6 +875,101 @@ class RoundTripToJaxTest(tf_test_util.JaxToTfTestCase):
     res = tf.function(converted_fun, jit_compile=True, autograph=False)(x, y)
     self.assertAllClose(expected, res.numpy(), atol=1e-5, rtol=1e-5)
 
+  @parameterized.named_parameters(
+      dict(
+          testcase_name=f"_{dtype.__name__}",
+          dtype=dtype,
+      )
+      for dtype in set(jtu.dtypes.all_floating)
+  )
+  def test_all_floating_input_gradient(self, dtype):
+    def tf_f(x):
+      res = tf.math.sin(x)
+      return tf.reduce_sum(res)
+
+    jax_f = jax2tf.call_tf(tf_f)
+    tf_f_rt = jax2tf.convert(jax_f)
+    x = jnp.array([5.0, 6.0, 7.0]).astype(dtype)
+
+    def assert_all_close_support_bfloat16(baseline, candidate):
+      def conversion(x):
+        # convert scalar to array and bfloat16 to float32
+        # to support self.assertAllClose numpy array comparision.
+        if x.shape == tf.TensorShape([]):
+          x = tf.convert_to_tensor([x])
+        if dtype == jnp.float16:
+          x = tf.cast(x, tf.float32)
+        return x
+
+      baseline = jax.tree_util.tree_map(conversion, baseline)
+      candidate = jax.tree_util.tree_map(conversion, candidate)
+      self.assertAllClose(baseline, candidate)
+
+    # Eager mode
+    assert_all_close_support_bfloat16(tf_f(x), tf_f_rt(x))
+
+    # Compiled function mode
+    assert_all_close_support_bfloat16(
+        tf.function(tf_f)(x), tf.function(tf_f_rt)(x)
+    )
+
+    # Compiled fucntion mode with jit_compiled=True
+    assert_all_close_support_bfloat16(
+        tf.function(tf_f, jit_compile=True)(x),
+        tf.function(tf_f_rt, jit_compile=True)(x),
+    )
+
+    # RoundTrip test for the gradient
+    grad_fun_jax = jax.grad(jax2tf.call_tf(tf_f))
+    grad_fun_jax_rt = jax2tf.call_tf(jax2tf.convert(grad_fun_jax))
+
+    # Eager mode
+    assert_all_close_support_bfloat16(grad_fun_jax(x), grad_fun_jax_rt(x))
+
+    # Jit mode
+    assert_all_close_support_bfloat16(
+        jax.jit(grad_fun_jax)(x), jax.jit(grad_fun_jax_rt)(x)
+    )
+
+  @parameterized.named_parameters(
+      dict(
+          testcase_name=f"_{dtype.__name__}",
+          dtype=dtype,
+      )
+      for dtype in set(jtu.dtypes.complex)
+  )
+  def test_complex_input_gradient(self, dtype):
+    def tf_f(x):
+      res = tf.math.sin(x)
+      return tf.reduce_sum(res)
+
+    x = jnp.array([(5.0 + 4.0j), (6.0 + 3.0j), (7.0 + 8.0j)]).astype(dtype)
+
+    jax_f = jax2tf.call_tf(tf_f)
+    tf_f_rt = jax2tf.convert(jax_f)
+
+    # Eager mode
+    self.assertAllClose(tf_f(x), tf_f_rt(x))
+
+    # tf.function context
+    self.assertAllClose(tf.function(tf_f)(x), tf.function(tf_f_rt)(x))
+
+    # tf.function context with jit_compiled=True
+    self.assertAllClose(
+        tf.function(tf_f, jit_compile=True)(x),
+        tf.function(tf_f_rt, jit_compile=True)(x),
+    )
+
+    # RoundTrip test for the gradient
+    grad_fun_jax = jax.grad(jax2tf.call_tf(tf_f), holomorphic=True)
+    grad_fun_jax_rt = jax2tf.call_tf(jax2tf.convert(grad_fun_jax))
+
+    # Eager mode
+    self.assertAllClose(grad_fun_jax(x), grad_fun_jax_rt(x))
+
+    # Jit mode
+    self.assertAllClose(jax.jit(grad_fun_jax)(x), jax.jit(grad_fun_jax_rt)(x))
+
 
 class RoundTripToTfTest(tf_test_util.JaxToTfTestCase):
   "Reloading output of call_tf into TF with jax2tf."
@@ -971,8 +1109,29 @@ class RoundTripToTfTest(tf_test_util.JaxToTfTestCase):
     ):
       fun_tf_rt(x)
 
+  def test_inner_native_lowering(self):
+    # Two nested jax2tf, the inner one being with native lowering
+    x = np.ones((3,), dtype=np.float32)
+    def f_inner_jax(x):
+      return jnp.sin(x)
+    def f_outer_jax(x):
+      f_inner_tf = jax2tf.convert(f_inner_jax, experimental_native_lowering=True)
+      return jnp.cos(jax2tf.call_tf(f_inner_tf)(x))
+
+    f_outer_tf = tf.function(
+        jax2tf.convert(f_outer_jax, experimental_native_lowering=False),
+        autograph=False)
+    f_outer_graph = str(f_outer_tf.get_concrete_function(tf.convert_to_tensor(x)).graph.as_graph_def())
+    # Quick way to check that there is an XlaCallModule op, and a Cos op, but no Sin op
+    self.assertIn('op: "Cos"', f_outer_graph)
+    self.assertIn('op: "XlaCallModule"', f_outer_graph)
+    self.assertNotIn('op: "Sin"', f_outer_graph)
+
   @_parameterized_jit
   def test_shape_polymorphism_static_output_shape(self, with_jit=True):
+    # TODO(b/268386622) Dynamic shapes not yet supported.
+    if config.jax2tf_default_experimental_native_lowering:
+      raise unittest.SkipTest("Skip test because of dynamic shapes.")
     x = np.array([0.7, 0.8], dtype=np.float32)
 
     def fun_tf(x):
