@@ -133,7 +133,8 @@ class Jax2TfLimitation(primitive_harness.Limitation):
       "eq", "floor", "gather", "ge", "gt", "imag", "iota", "iota_2x32_shape",
       "is_finite", "le", "logistic", "lt", "log", "mul", "ne", "neg", "not",
       "or", "pad", "population_count", "random_categorical", "random_uniform",
-      "random_randint", "reduce", "reduce_and", "reduce_prod", "reduce_or",
+      "random_randint", "reduce", "reduce_and", "reduce_precision",
+      "reduce_prod", "reduce_or",
       "reduce_sum", "reduce_window_mul", "reduce_window_min",
       "reduce_window_max", "real", "reshape", "rev", "rsqrt", "select_n",
       "select_and_scatter_add", "shift_left", "shift_right_logical",
