@@ -35,7 +35,7 @@ from jax.experimental import jax2tf
 from jax.experimental.jax2tf.tests import tf_test_util
 from jax.experimental import pjit
 from jax.interpreters import mlir
-from jax.interpreters.pxla import PartitionSpec as P
+from jax.sharding import PartitionSpec as P
 
 import numpy as np
 import tensorflow as tf  # type: ignore[import]
@@ -976,7 +976,7 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
     self.assertLen(f_tf_graph_consts, 1)
 
   def test_shared_constants_randint(self):
-    # randint has the property that that the TF lowering of the randbits_p
+    # randint has the property that the TF lowering of the randbits_p
     # primitive generates constants that did not exist in the Jaxpr. As such
     # it has created new errors related to the sharing of the constants.
     if config.jax2tf_default_experimental_native_lowering:
