@@ -59,20 +59,20 @@ from functools import partial
 import numpy as np
 
 import jax
-from jax import core
 from jax import lax
 from jax.interpreters import xla
 import jax.numpy as jnp
 from jax.experimental import pjit
 from jax.interpreters import partial_eval as pe, pxla
-from jax._src.api_util import shaped_abstractify
 from jax.tree_util import (register_pytree_node, tree_structure,
                            treedef_is_leaf, tree_flatten, tree_unflatten,)
 
 from jax._src import ad_util
+from jax._src import core
 from jax._src import dispatch
-from jax._src.lax import lax as lax_internal
 from jax._src import linear_util as lu
+from jax._src.api_util import shaped_abstractify
+from jax._src.lax import lax as lax_internal
 from jax._src.util import unzip2, weakref_lru_cache
 
 
