@@ -21,15 +21,16 @@ import zlib
 from typing import Any
 import jax
 from jax.tree_util import tree_flatten, tree_map, tree_unflatten
+from jax._src import core
 from jax._src import dispatch
 from jax._src import array
 from jax._src import sharding
 from jax.tree_util import PyTreeDef
 from jax._src.interpreters import pxla
 from jax.interpreters import xla
-from jax.experimental import pjit as pjit_lib
+from jax._src import pjit as pjit_lib
 from jax.experimental.pjit import pjit, FROM_GDA
-from jax.interpreters.pxla import PartitionSpec as P
+from jax.sharding import PartitionSpec as P
 from jax.experimental.global_device_array import GlobalDeviceArray
 from jax._src import distributed
 from jax._src import config as config_internal
@@ -113,7 +114,7 @@ def _handle_array_process_allgather(inp, tiled):
     if host_np_arr.ndim == 0 or not tiled:
       host_np_arr = np.expand_dims(host_np_arr, axis=0)
 
-    aval = jax.core.ShapedArray(host_np_arr.shape, host_np_arr.dtype)
+    aval = core.ShapedArray(host_np_arr.shape, host_np_arr.dtype)
     global_aval = global_mesh._local_to_global(
         pxla.get_array_mapping(pspec), aval)
 
@@ -325,7 +326,7 @@ def host_local_array_to_global_array(local_inputs: Any,
       ))
 
     global_aval = _local_to_global_aval(
-        jax.core.ShapedArray(arr.shape, arrays[0].dtype), global_mesh, pspec)
+        core.ShapedArray(arr.shape, arrays[0].dtype), global_mesh, pspec)
 
     return array.ArrayImpl(
         global_aval, jax.sharding.NamedSharding(global_mesh, pspec),
