@@ -24,23 +24,24 @@ from typing import (Any, Callable, Dict, Hashable, List, Optional, Sequence,
 import numpy as np
 
 import jax
-from jax import core
-from jax.core import Tracer
 from jax.sharding import NamedSharding, PartitionSpec, Mesh
+from jax._src import core
 from jax._src import ad_util
+from jax._src import custom_derivatives
 from jax._src import linear_util as lu
 from jax._src import ops
 from jax._src import pjit
 from jax._src import source_info_util
 from jax._src import traceback_util
 from jax._src import util
+from jax._src.core import Tracer
 from jax._src.lax import (lax, parallel as lax_parallel, slicing,
                           windowed_reductions, fft, linalg)
 from jax._src.util import (prod, HashableFunction, unzip2, as_hashable_function,
                            memoize, partition_list, merge_lists)
 from jax.api_util import flatten_fun_nokwargs, shaped_abstractify
 from jax.interpreters import batching
-from jax.interpreters import mlir
+from jax._src.interpreters import mlir
 from jax.interpreters import partial_eval as pe
 from jax.interpreters import xla
 from jax._src.interpreters import pxla
@@ -371,7 +372,8 @@ def _shard_map_staging(
   with core.extend_axis_env_nd(mesh.shape.items()):
     jaxpr = pe.convert_constvars_jaxpr(jaxpr)
   params = dict(mesh=mesh, in_names=in_names_staged,
-                out_names=out_names_thunk(), jaxpr=jaxpr, check_rep=check_rep)
+                out_names=tuple(out_names_thunk()), jaxpr=jaxpr,
+                check_rep=check_rep)
   eqn = pe.new_jaxpr_eqn([*constvars, *invars], outvars, prim, params,
                          jaxpr.effects, source_info)
   trace.frame.add_eqn(eqn)
@@ -491,7 +493,7 @@ def _shard_map_impl(trace, prim, fun, args, *, mesh, in_names, out_names_thunk,
   del prim
   args = map(partial(_unmatch_spec, mesh), in_names, args)
   in_rep = map(partial(_in_names_to_rep, mesh), in_names)
-  with core.new_base_main(ShardMapTrace, mesh=mesh) as main:
+  with core.new_base_main(ShardMapTrace, mesh=mesh, check=check_rep) as main:
     with core.new_sublevel(), core.extend_axis_env_nd(mesh.shape.items()):
       t = main.with_cur_sublevel()
       in_tracers = map(partial(ShardMapTracer, t), in_rep, args)
@@ -500,7 +502,7 @@ def _shard_map_impl(trace, prim, fun, args, *, mesh, in_names, out_names_thunk,
       outs_, out_rep = unzip2((t.val, t.rep) for t in out_tracers)
       del main, t, in_tracers, ans, out_tracers
   out_avals = [core.mapped_aval(x.shape[0], 0, core.get_aval(x)) for x in outs_]
-  _check_names(out_names_thunk(), out_avals)
+  _check_names(out_names_thunk(), out_avals)  # pytype: disable=wrong-arg-types
   if check_rep: _check_reps(mesh, out_names_thunk(), out_rep)
   return map(partial(_match_spec, mesh), out_rep, out_names_thunk(), outs_)
 core.EvalTrace.process_shard_map = _shard_map_impl
@@ -546,10 +548,12 @@ def _add_singleton(x): return x.reshape(1, *x.shape)
 
 class ShardMapTrace(core.Trace):
   mesh: Mesh
+  check: bool
 
-  def __init__(self, *args, mesh):
+  def __init__(self, *args, mesh, check):
     super().__init__(*args)
     self.mesh = mesh
+    self.check = check
 
   def pure(self, val):
     val_ = _unmatch_spec(self.mesh, {}, val)
@@ -564,7 +568,7 @@ class ShardMapTrace(core.Trace):
     with core.eval_context(), jax.disable_jit(False):
       out_vals = jax.jit(f)(*in_vals)
     rule = _rep_rules.get(prim, partial(_rep_rule, prim))
-    out_rep = rule(self.mesh, *in_rep, **params)
+    out_rep = rule(self.mesh, *in_rep, **params) if self.check else set()
     if prim.multiple_results:
       out_rep = [out_rep] * len(out_vals) if type(out_rep) is set else out_rep
       return map(partial(ShardMapTracer, self), out_rep, out_vals)
@@ -575,10 +579,13 @@ class ShardMapTrace(core.Trace):
     fun, jaxpr = _grab_jaxpr_shadily(fun)  # TODO remove with initial-style jit
     bind = partial(call_primitive.bind, fun)  # TODO caching (compat w/ jaxpr())
     fake_primitive = pxla.FakePrimitive(multiple_results=True, bind=bind)
-    _rep_rules[fake_primitive] = lambda *_, **__: set()
+    _rep_rules[fake_primitive] = lambda *_, **__: set()  # pytype: disable=container-type-mismatch
     out_tracers_ = self.process_primitive(fake_primitive, tracers, params)
     out_vals = [t.val for t in out_tracers_]
-    out_rep = _output_rep(self.mesh, jaxpr(), [t.rep for t in tracers])
+    if self.check:
+      out_rep = _output_rep(self.mesh, jaxpr(), [t.rep for t in tracers])
+    else:
+      out_rep = [set()] * len(out_vals)
     return map(partial(ShardMapTracer, self), out_rep, out_vals)
 
 @lu.transformation_with_aux
@@ -624,11 +631,13 @@ def _prim_applier(prim, params_tup, mesh, *args):
   def apply(*args):
     outs = prim.bind(*map(_rem_singleton, args), **dict(params_tup))
     return tree_map(_add_singleton, outs)
-  return shard_map(apply, mesh, P(mesh.axis_names), P(mesh.axis_names))(*args)
+  spec = P(mesh.axis_names)
+  return shard_map(apply, mesh, spec, spec, False)(*args)
 
 # Static replication checking
 
-def _rep_rule(prim, mesh, *in_rep, **params):
+def _rep_rule(prim: core.Primitive, mesh: Mesh, *in_rep: Set[AxisName],
+              **params: Any) -> Union[Set[AxisName], List[Set[AxisName]]]:
   raise NotImplementedError(f"no replication rule for {prim}")
 
 _rep_rules: Dict[core.Primitive, Callable] = {}
@@ -636,14 +645,16 @@ register_rule = lambda prim: lambda rule: _rep_rules.setdefault(prim, rule)
 register_standard = lambda prim: _rep_rules.setdefault(prim, _standard_rep_rule)
 
 def _standard_rep_rule(_, *in_rep, **__):
-  return set.intersection(*in_rep)
+  return set.intersection(*in_rep) if in_rep else set()
 
 for o in it.chain(lax.__dict__.values(), slicing.__dict__.values(),
                   windowed_reductions.__dict__.values(), fft.__dict__.values(),
-                  linalg.__dict__.values(), ops.__dict__.values()):
+                  linalg.__dict__.values(), ops.__dict__.values(),
+                  ad_util.__dict__.values(),
+                  custom_derivatives.__dict__.values()):
   if isinstance(o, core.Primitive): register_standard(o)
-register_standard(ad_util.add_any_p)
 
+register_standard(xla.xla_call_p)
 register_standard(lax_parallel.ppermute_p)  # doesn't change replication
 
 @register_rule(lax_parallel.psum_p)
@@ -694,18 +705,25 @@ def _shard_map_batch(
   if all(bdim is batching.not_mapped for bdim in in_dims):
     return prim.bind(fun, *in_vals, mesh=mesh, in_names=in_names,
                      out_names_thunk=out_names_thunk, check_rep=check_rep)
-  if trace.spmd_axis_name is not None:
-    raise NotImplementedError  # TODO add named axis to specs
   if any(isinstance(d, batching.ConcatAxis) for d in in_dims):
     raise NotImplementedError
   fun, out_dims = batching.batch_subtrace(fun, trace.main, tuple(in_dims))
-  new_in_names = [{ax + (d is not batching.not_mapped and ax <= d): names[ax]  # type: ignore
+  new_in_names = [{ax + (d is not batching.not_mapped and d <= ax): names[ax]  # type: ignore
                    for ax in names} for names, d in zip(in_names, in_dims)]
+  spmd_axis_name = trace.spmd_axis_name
+  if spmd_axis_name is not None:
+    new_in_names = [{**ns, d:spmd_axis_name} if d is not batching.not_mapped  # type: ignore
+                    else ns for ns, d in zip(new_in_names, in_dims)]
   @as_hashable_function(closure=out_names_thunk)
   def new_out_names_thunk():
     out_names = out_names_thunk()
-    return [{ax + (d is not batching.not_mapped and ax <= d): names[ax]
-             for ax in names} for names, d in zip(out_names, out_dims())]
+    out_names_ = [{ax + (d is not batching.not_mapped and d <= ax): names[ax]
+                   for ax in names} for names, d in zip(out_names, out_dims())]
+    if spmd_axis_name is not None:
+      out_names_ = [{**ns, d:spmd_axis_name} if d is not batching.not_mapped
+                    else ns for ns, d in zip(out_names_, out_dims())]
+    return out_names_
+
   new_params = dict(mesh=mesh, in_names=new_in_names,
                     out_names_thunk=new_out_names_thunk, check_rep=check_rep)
   out_vals = prim.bind(fun, *in_vals, **new_params)
@@ -908,6 +926,39 @@ def _shard_map_axis_subst(params, subst, traverse):
   return dict(params, jaxpr=new_jaxpr)
 core.axis_substitution_rules[shard_map_p] = _shard_map_axis_subst
 
+# Remat
+
+def _pe_custom_params(
+    unks_in: List[bool], inst_in: List[bool], kept_outs_known: List[bool],
+    kept_outs_staged: List[bool], num_res: int, params_known: Dict[str, Any],
+    params_staged: Dict[str, Any]
+  ) -> Tuple[Dict[str, Any], Dict[str, Any]]:
+  # prune inputs to jaxpr_known according to unks_in
+  mesh = params_known['mesh']
+  in_names_known, _ = partition_list(unks_in, params_known['in_names'])
+  _, out_names_known = partition_list(kept_outs_known, params_known['out_names'])
+  out_names_known = out_names_known + [{0: (*mesh.axis_names,)}] * num_res
+  new_params_known = dict(params_known, in_names=tuple(in_names_known),
+                          out_names=tuple(out_names_known))
+
+  # added num_res new inputs to jaxpr_staged, pruning according to inst_in
+  _, in_names_staged = partition_list(inst_in, params_staged['in_names'])
+  in_names_staged = [{0: (*mesh.axis_names,)}] * num_res + in_names_staged
+  _, out_names_staged = partition_list(kept_outs_staged, params_staged['out_names'])
+  new_params_staged = dict(params_staged, in_names=tuple(in_names_staged),
+                           out_names=tuple(out_names_staged), check_rep=False)
+  return new_params_known, new_params_staged
+
+def _pe_custom_res(params_known, aval):
+  mesh = params_known['mesh']
+  return _unshard_aval(mesh, {0: (*mesh.axis_names,)}, aval)
+
+pe.partial_eval_jaxpr_custom_rules[shard_map_p] = \
+    partial(pe.call_partial_eval_custom_rule, 'jaxpr', _pe_custom_params,
+            res_aval=_pe_custom_res)
+
+# Misc
+
 # TODO(mattjj): move this to _src/util.py
 class HashablePartial:
   def __init__(self, f, *args, **kwargs):
