@@ -17,12 +17,15 @@ from __future__ import annotations
 
 from functools import partial
 import operator
+import warnings
 
 from typing import NamedTuple, Optional, Sequence, Tuple, Union
 
 import numpy as np
 
-from jax import core
+import jax
+import jax.numpy as jnp
+from jax import config
 from jax import lax
 from jax import tree_util
 from jax.experimental.sparse._base import JAXSparse
@@ -30,13 +33,17 @@ from jax.experimental.sparse import bcoo
 from jax.experimental.sparse.util import (
     nfold_vmap, _count_stored_elements,
     _csr_to_coo, _dot_general_validated_shape,
-    SparseInfo, Shape)
-import jax.numpy as jnp
-from jax._src import api_util
-from jax._src.lax.lax import DotDimensionNumbers
+    CuSparseEfficiencyWarning, SparseInfo, Shape)
 from jax.util import split_list, safe_zip
-from jax.interpreters import batching
-from jax.interpreters import mlir
+
+from jax._src import api_util
+from jax._src import core
+from jax._src.lax.lax import DotDimensionNumbers, _dot_general_batch_dim_nums
+from jax._src.lib import gpu_sparse
+from jax._src.lib.mlir.dialects import hlo
+from jax._src.interpreters import ad
+from jax._src.interpreters import batching
+from jax._src.interpreters import mlir
 from jax._src.typing import Array, ArrayLike, DTypeLike
 
 
@@ -63,6 +70,24 @@ def bcsr_sum_duplicates(mat: BCSR, nse: Optional[int] = None) -> BCSR:
   return BCSR.from_bcoo(bcoo.bcoo_sum_duplicates(mat.to_bcoo(), nse=nse))
 
 
+def _bcsr_batch_dims_to_front(batched_args, batch_dims, spinfo, batch_size=None):
+  data, indices, indptr = batched_args
+  data_bdim, indices_bdim, indptr_bdim = batch_dims
+  n_batch = indices.ndim - 1 + int(indices_bdim is None)
+  if not all(b is None or 0 <= b < n_batch for b in batch_dims):
+    raise NotImplementedError("batch_dims must be None or satisfy 0 < dim < n_batch. "
+                              f"Got {batch_dims=} for {n_batch=}.")
+  batched_data, batched_indices, batched_indptr = [
+      lax.expand_dims(arg, [0]) if bdim is None else jnp.moveaxis(arg, bdim, 0)
+      for arg, bdim in [(data, data_bdim), (indices, indices_bdim), (indptr, indptr_bdim)]]
+  if batch_size is None:
+    batch_size = max(arg.shape[dim] for arg, dim in zip(batched_args, batch_dims) if dim is not None)
+  batched_spinfo = SparseInfo((batch_size, *spinfo.shape),
+                              indices_sorted=spinfo.indices_sorted,
+                              unique_indices=spinfo.unique_indices)
+  return batched_data, batched_indices, batched_indptr, batched_spinfo
+
+
 class BCSRProperties(NamedTuple):
   n_batch: int
   n_dense: int
@@ -73,7 +98,7 @@ def _compatible(shape1: Sequence[int], shape2: Sequence[int]) -> bool:
   return all(s1 in (1, s2) for s1, s2 in safe_zip(shape1, shape2))
 
 
-def _validate_bcsr_indices(indices: jnp.ndarray, indptr: jnp.ndarray,
+def _validate_bcsr_indices(indices: jax.Array, indptr: jax.Array,
                            shape: Sequence[int]) -> BCSRProperties:
   assert jnp.issubdtype(indices.dtype, jnp.integer)
   assert jnp.issubdtype(indptr.dtype, jnp.integer)
@@ -94,8 +119,8 @@ def _validate_bcsr_indices(indices: jnp.ndarray, indptr: jnp.ndarray,
   return BCSRProperties(n_batch=n_batch, n_dense=n_dense, nse=nse)
 
 
-def _validate_bcsr(data: jnp.ndarray, indices: jnp.ndarray,
-                   indptr: jnp.ndarray, shape: Sequence[int]) -> BCSRProperties:
+def _validate_bcsr(data: jax.Array, indices: jax.Array,
+                   indptr: jax.Array, shape: Sequence[int]) -> BCSRProperties:
   props = _validate_bcsr_indices(indices, indptr, shape)
   shape = tuple(shape)
   n_batch, n_dense, nse = props.n_batch, props.n_dense, props.nse
@@ -110,8 +135,8 @@ def _validate_bcsr(data: jnp.ndarray, indices: jnp.ndarray,
   return props
 
 
-def _bcsr_to_bcoo(indices: jnp.ndarray, indptr: jnp.ndarray, *,
-                  shape: Sequence[int]) -> jnp.ndarray:
+def _bcsr_to_bcoo(indices: jax.Array, indptr: jax.Array, *,
+                  shape: Sequence[int]) -> jax.Array:
   """Given BCSR (indices, indptr), return BCOO (indices)."""
   n_batch, _, _ = _validate_bcsr_indices(indices, indptr, shape)
   csr_to_coo = nfold_vmap(_csr_to_coo, n_batch)
@@ -231,17 +256,45 @@ def _bcsr_fromdense_abstract_eval(mat, *, nse, n_batch, n_dense, index_dtype):
 def _bcsr_fromdense_batching_rule(batched_args, batch_dims, *, nse, n_batch,
                                   n_dense, index_dtype):
   M, = batched_args
-  if batch_dims != (0,):
-    raise NotImplementedError(f"{batch_dims=}")
-  new_n_batch = n_batch + 1
-  n_sparse = M.ndim - n_dense - new_n_batch
-  if n_sparse != 2:
-    raise ValueError("_bcsr_fromdense_batching_rule: must have 2 sparse "
-                     f"dimensions but {n_sparse} is given.")
-  return _bcsr_fromdense(M, nse=nse, n_batch=new_n_batch, n_dense=n_dense,
-                         index_dtype=index_dtype), (0, 0, 0)
+  bdim, = batch_dims
+  if not (0 <= bdim <= n_batch):
+    raise ValueError(f"Expected 0 < bdim <= n_batch; got {bdim=}, {n_batch=}")
+  return _bcsr_fromdense(M, nse=nse, n_batch=n_batch + 1, n_dense=n_dense, index_dtype=index_dtype), (bdim, bdim, bdim)
 
 
+def _bcsr_fromdense_jvp(primals, tangents, *, nse, n_batch, n_dense, index_dtype):
+  M, = primals
+  Mdot, = tangents
+
+  primals_out = _bcsr_fromdense(M, nse=nse, n_batch=n_batch, n_dense=n_dense, index_dtype=index_dtype)
+  data, indices, indptr = primals_out
+
+  if type(Mdot) is ad.Zero:
+    data_dot = ad.Zero.from_value(data)
+  else:
+    data_dot = bcsr_extract(indices, indptr, Mdot)
+
+  tangents_out = (data_dot, ad.Zero.from_value(indices), ad.Zero.from_value(indptr))
+
+  return primals_out, tangents_out
+
+
+def _bcsr_fromdense_transpose(ct, M, *, nse, n_batch, n_dense, index_dtype):
+  data, indices, indptr = ct
+  n_sparse = M.ndim - n_batch - n_dense
+  assert data.shape == M.shape[:n_batch] + (nse,) + M.shape[n_batch + n_sparse:]
+  assert indices.shape == M.shape[:n_batch] + (n_sparse, nse)
+  assert indptr.shape == M.shape[:n_batch] + (M.shape[n_batch] + 1,)
+  assert indices.dtype == index_dtype
+  assert indptr.dtype == index_dtype
+  if isinstance(indices, ad.Zero) or isinstance(indptr, ad.Zero):
+    raise ValueError("Cannot transpose with respect to sparse indices")
+  assert ad.is_undefined_primal(M)
+  return _bcsr_todense(data, indices, indptr, spinfo=SparseInfo(M.aval.shape))
+
+
+ad.primitive_jvps[bcsr_fromdense_p] = _bcsr_fromdense_jvp
+ad.primitive_transposes[bcsr_fromdense_p] = _bcsr_fromdense_transpose
 batching.primitive_batchers[bcsr_fromdense_p] = _bcsr_fromdense_batching_rule
 mlir.register_lowering(bcsr_fromdense_p, mlir.lower_fun(
     _bcsr_fromdense_impl, multiple_results=True))
@@ -298,17 +351,27 @@ def _bcsr_todense_abstract_eval(data, indices, indptr, *, spinfo):
 
 
 def _bcsr_todense_batching_rule(batched_args, batch_dims, *, spinfo):
-  data, indices, indptr = batched_args
-  if any(b not in [0, None] for b in batch_dims):
-    raise NotImplementedError(f"{batch_dims=}. Only 0 and None are supported.")
-  if batch_dims[0] is None:
-    data = data[None, ...]
-  if batch_dims[1] is None:
-    indices = indices[None, ...]
-  if batch_dims[2] is None:
-    indptr = indptr[None, ...]
+  data, indices, indptr, spinfo = _bcsr_batch_dims_to_front(batched_args, batch_dims, spinfo)
   return _bcsr_todense(data, indices, indptr, spinfo=spinfo), 0
 
+
+def _bcsr_todense_jvp(data_dot, data, indices, indptr, *, spinfo):
+  del data
+  return _bcsr_todense(data_dot, indices, indptr, spinfo=spinfo)
+
+
+def _bcsr_todense_transpose(ct, data, indices, indptr, *, spinfo):
+  shape = spinfo.shape
+  assert ad.is_undefined_primal(data)
+  if ad.is_undefined_primal(indices) or ad.is_undefined_primal(indptr):
+    raise ValueError("Cannot transpose with respect to sparse indices")
+  assert ct.shape == shape
+  assert ct.dtype == data.aval.dtype
+  return bcsr_extract(indices, indptr, ct), indices, indptr
+
+
+ad.defjvp(bcsr_todense_p, _bcsr_todense_jvp, None, None)
+ad.primitive_transposes[bcsr_todense_p] = _bcsr_todense_transpose
 batching.primitive_batchers[bcsr_todense_p] = _bcsr_todense_batching_rule
 mlir.register_lowering(bcsr_todense_p, mlir.lower_fun(
     _bcsr_todense_impl, multiple_results=False))
@@ -347,6 +410,43 @@ def _bcsr_extract_abstract_eval(indices, indptr, mat):
   return core.ShapedArray(out_shape, mat.dtype)
 
 
+def _bcsr_extract_jvp(arr_dot, indices, indptr, arr):
+  assert arr_dot.shape == arr.shape
+  return bcsr_extract(indices, indptr, arr_dot)
+
+
+def _bcsr_extract_transpose(ct, indices, indptr, arr):
+  assert ad.is_undefined_primal(arr)
+  if ad.is_undefined_primal(indices) or ad.is_undefined_primal(indptr):
+    raise ValueError("Cannot transpose with respect to sparse indices")
+  assert ct.dtype == arr.aval.dtype
+  return indices, indptr, _bcsr_todense(ct, indices, indptr, spinfo=SparseInfo(arr.aval.shape))
+
+
+def _bcsr_extract_batching_rule(batched_args, batch_dims):
+  indices, indptr, arr = batched_args
+  bdim_set = {b for b in batch_dims if b is not None}
+  if len(bdim_set) != 1:
+    # TODO(jakevdp): handle this by moving bdim to front?
+    raise NotImplementedError("bcoo_extract with unequal batch dimensions.")
+  bdim = next(iter(bdim_set))
+  if batch_dims[0] is None:
+    indices = lax.expand_dims(indices, (bdim,))
+  if batch_dims[1] is None:
+    indptr = lax.expand_dims(indptr, (bdim,))
+  if batch_dims[2] is None:
+    # TODO(jakevdp) can we handle this case without explicit broadcasting?
+    result_shape = list(arr.shape)
+    result_shape.insert(bdim, indices.shape[bdim])
+    arr = lax.broadcast_in_dim(arr, result_shape, (bdim,))
+  n_batch = indices.ndim - 1
+  if bdim >= n_batch:
+    raise ValueError(f"{batch_dims=} out of range for indices with {n_batch=}")
+  return bcsr_extract(indices, indptr, arr), bdim
+
+ad.defjvp(bcsr_extract_p, None, None, _bcsr_extract_jvp)
+ad.primitive_transposes[bcsr_extract_p] = _bcsr_extract_transpose
+batching.primitive_batchers[bcsr_extract_p] = _bcsr_extract_batching_rule
 mlir.register_lowering(bcsr_extract_p, mlir.lower_fun(
     _bcsr_extract_impl, multiple_results=False))
 
@@ -379,8 +479,8 @@ def bcsr_dot_general(lhs: Union[BCSR, Array], rhs: Array, *,
     dense, the result will be dense, of type ndarray.
   """
   del precision, preferred_element_type  # unused
-  if isinstance(rhs, (np.ndarray, jnp.ndarray)):
-    if isinstance(lhs, (np.ndarray, jnp.ndarray)):
+  if isinstance(rhs, (np.ndarray, jax.Array)):
+    if isinstance(lhs, (np.ndarray, jax.Array)):
       return lax.dot_general(lhs, rhs, dimension_numbers=dimension_numbers)
 
     if isinstance(lhs, BCSR):
@@ -393,8 +493,8 @@ def bcsr_dot_general(lhs: Union[BCSR, Array], rhs: Array, *,
                             "lhs and ndarray rhs.")
 
 
-def _bcsr_dot_general(lhs_data: jnp.ndarray, lhs_indices: jnp.ndarray,
-                      lhs_indptr: jnp.ndarray, rhs: Array, *,
+def _bcsr_dot_general(lhs_data: jax.Array, lhs_indices: jax.Array,
+                      lhs_indptr: jax.Array, rhs: Array, *,
                       dimension_numbers: DotDimensionNumbers,
                       lhs_spinfo: SparseInfo) -> Array:
   (lhs_contract, rhs_contract), (lhs_batch, rhs_batch) = dimension_numbers
@@ -448,62 +548,186 @@ def _bcsr_dot_general_abstract_eval(lhs_data, lhs_indices, lhs_indptr, rhs, *,
   return core.ShapedArray(out_shape, lhs_data.dtype)
 
 
-# def _bcsr_dot_general_jvp_lhs(lhs_data_dot, lhs_data, lhs_indices, lhs_indptr,
-#                               rhs, *, dimension_numbers, lhs_spinfo):
-#   del lhs_data
-#   return _bcsr_dot_general(lhs_data_dot, lhs_indices, lhs_indptr, rhs,
-#                            dimension_numbers=dimension_numbers,
-#                            lhs_spinfo=lhs_spinfo)
+def _bcsr_dot_general_jvp_lhs(lhs_data_dot, lhs_data, lhs_indices, lhs_indptr, rhs, *,
+                              dimension_numbers, lhs_spinfo):
+  del lhs_data
+  return _bcsr_dot_general(lhs_data_dot, lhs_indices, lhs_indptr, rhs,
+                           dimension_numbers=dimension_numbers,
+                           lhs_spinfo=lhs_spinfo)
 
 
-# def _bcsr_dot_general_jvp_rhs(rhs_dot, lhs_data, lhs_indices, lhs_indptr, rhs,
-#                               *, dimension_numbers, lhs_spinfo):
-#   del rhs
-#   return _bcsr_dot_general(lhs_data, lhs_indices, lhs_indptr, rhs_dot,
-#                            dimension_numbers=dimension_numbers,
-#                            lhs_spinfo=lhs_spinfo)
+def _bcsr_dot_general_jvp_rhs(rhs_dot, lhs_data, lhs_indices, lhs_indptr, rhs, *,
+                              dimension_numbers, lhs_spinfo):
+  del rhs
+  return _bcsr_dot_general(lhs_data, lhs_indices, lhs_indptr, rhs_dot,
+                           dimension_numbers=dimension_numbers,
+                           lhs_spinfo=lhs_spinfo)
 
 
-# def _bcsr_dot_general_transpose(ct, lhs_data, lhs_indices, lhs_inptr, rhs, *,
-#                                  dimension_numbers, lhs_spinfo):
-#   lhs_bcoo_indices = _bcsr_to_bcoo(
-#     lhs_indices, lhs_inptr, shape=lhs_spinfo.shape)
-#   return bcoo._bcoo_dot_general_transpose(
-#       ct, lhs_data, lhs_bcoo_indices, rhs, dimension_numbers=dimension_numbers,
-#       lhs_spinfo=lhs_spinfo)
+def _bcsr_dot_general_transpose(ct, lhs_data, lhs_indices, lhs_indptr, rhs, *,
+                                 dimension_numbers, lhs_spinfo):
+  # TODO(jakevdp): implement this in terms of bcsr_dot_general
+  lhs_bcoo_indices = _bcsr_to_bcoo(
+    lhs_indices, lhs_indptr, shape=lhs_spinfo.shape)
+  data_out, _, rhs_out = bcoo._bcoo_dot_general_transpose(
+      ct, lhs_data, lhs_bcoo_indices, rhs, dimension_numbers=dimension_numbers,
+      lhs_spinfo=lhs_spinfo)
+  return data_out, lhs_indices, lhs_indptr, rhs_out
 
 
-# def _bcsr_dot_general_batch_rule(batched_args, batch_dims, *,
-#                                  dimension_numbers, lhs_spinfo):
-#   lhs_data, lhs_indices, lhs_indptr, rhs = batched_args
-#   lhs_bcoo_indices = _bcsr_to_bcoo(
-#     lhs_indices, lhs_indptr, shape=lhs_spinfo.shape)
-#   return bcoo._bcoo_dot_general_batch_rule(
-#       (lhs_data, lhs_bcoo_indices, rhs), batch_dims,
-#       dimension_numbers=dimension_numbers, lhs_spinfo=lhs_spinfo)
+def _bcsr_dot_general_batch_rule(batched_args, batch_dims, *,
+                                 dimension_numbers, lhs_spinfo):
+  *lhs_args, rhs = batched_args
+  *lhs_dims, rhs_bdim = batch_dims
+  *new_lhs_args, new_lhs_spinfo = _bcsr_batch_dims_to_front(
+    lhs_args, lhs_dims, lhs_spinfo,
+    batch_size=None if rhs_bdim is None else rhs.shape[rhs_bdim])
+  new_dimension_numbers, result_batch_dim = _dot_general_batch_dim_nums(
+      (len(lhs_spinfo.shape), rhs.ndim), (0, rhs_bdim), dimension_numbers)
+  batched_out = _bcsr_dot_general(*new_lhs_args, rhs, lhs_spinfo=new_lhs_spinfo,
+                                  dimension_numbers=new_dimension_numbers)
+  return batched_out, result_batch_dim
 
 
-# ad.defjvp(bcsr_dot_general_p, _bcsr_dot_general_jvp_lhs, None,
-#           _bcsr_dot_general_jvp_rhs)
-# ad.primitive_transposes[bcsr_dot_general_p] = _bcsr_dot_general_transpose
-# batching.primitive_batchers[bcsr_dot_general_p] = _bcsr_dot_general_batch_rule
+ad.defjvp(bcsr_dot_general_p, _bcsr_dot_general_jvp_lhs, None, None,
+          _bcsr_dot_general_jvp_rhs)
+ad.primitive_transposes[bcsr_dot_general_p] = _bcsr_dot_general_transpose
+batching.primitive_batchers[bcsr_dot_general_p] = _bcsr_dot_general_batch_rule
 
 
+def _bcsr_correct_out_of_bound_indices(data, indices, indptr, rhs, *, shape):
+  del rhs  # unused
+  props = _validate_bcsr(data, indices, indptr, shape)
+  if props.n_batch:
+    f = partial(_bcsr_correct_out_of_bound_indices, shape=shape[props.n_batch:])
+    return nfold_vmap(f, props.n_batch)(data, indices, indptr)
+  extent = indptr[-1]
+  i_data = lax.broadcasted_iota(indptr.dtype, data.shape, 0)
+  data = jnp.where(i_data < extent, data, 0)
+  i_indices = lax.broadcasted_iota(indptr.dtype, indices.shape, 0)
+  indices = jnp.where(i_indices < extent, indices, 0)
+  return [data, indices]
+
+_bcsr_correct_out_of_bound_indices_lowered = mlir.lower_fun(
+    _bcsr_correct_out_of_bound_indices, multiple_results=True)
+
+def _bcsr_dot_general_gpu_lowering(
+    csr_matvec_lowering, csr_matmat_lowering,
+    ctx, lhs_data, lhs_indices, lhs_indptr, rhs, *, dimension_numbers,
+    lhs_spinfo: SparseInfo):
+
+  if not config.jax_bcoo_cusparse_lowering:
+    return _bcsr_dot_general_default_lowering(
+      ctx, lhs_data, lhs_indices, lhs_indptr, rhs,
+      dimension_numbers=dimension_numbers, lhs_spinfo=lhs_spinfo)
+
+  (lhs_contract, rhs_contract), (lhs_batch, rhs_batch) = dimension_numbers
+  lhs_data_aval, lhs_indices_aval, lhs_indptr_aval, rhs_aval = ctx.avals_in
+  props = _validate_bcsr(
+      lhs_data_aval, lhs_indices_aval, lhs_indptr_aval, lhs_spinfo.shape)
+
+  use_default_lowering = False
+  dtype = lhs_data_aval.dtype
+  # TODO(vanderplas, tianjianlu): lower batched matmuls to GPU
+  if lhs_batch or rhs_batch:
+    # batch dimensions in dot_general are not supported
+    use_default_lowering = True
+  elif len(lhs_spinfo.shape) != 2 or rhs_aval.ndim not in [1, 2]:
+    # only matmat / matvec supported
+    use_default_lowering = True
+  elif props.n_batch or props.n_dense:
+    # batch and dense dimensions in BCSR not supported
+    use_default_lowering = True
+  elif list(lhs_contract) != [1]:
+    # cusparse cannot contract over more than one dimension
+    use_default_lowering = True
+  elif dtype not in [np.float32, np.float64, np.complex64, np.complex128]:
+    # This would be supported if not for the dtype.
+    warnings.warn(f'bcsr_dot_general cusparse/hipsparse lowering not available '
+                  f'for {dtype=}. Falling back to default implementation.',
+                  CuSparseEfficiencyWarning)
+    use_default_lowering = True
+
+  if use_default_lowering:
+    return _bcsr_dot_general_default_lowering(
+      ctx, lhs_data, lhs_indices, lhs_indptr, rhs,
+      dimension_numbers=dimension_numbers, lhs_spinfo=lhs_spinfo)
+
+  # Account for a bug in cusparse: it references indices and data beyond
+  # the extent of indptr.
+  (lhs_data,), (lhs_indices,) = _bcsr_correct_out_of_bound_indices_lowered(
+      ctx, lhs_data, lhs_indices, lhs_indptr, rhs, shape=lhs_spinfo.shape)
+
+  if rhs_aval.ndim == 1:
+    dot_general_fn = csr_matvec_lowering
+    x_dtype = 'x_dtype'
+  elif rhs_aval.ndim == 2:
+    dot_general_fn = csr_matmat_lowering
+    x_dtype = 'B_dtype'
+    if rhs_contract[0] == 1:
+      rhs = hlo.TransposeOp(
+          rhs, permutation=mlir.dense_int_elements([1, 0])).result
+  else:
+    raise ValueError(f"rhs has to be 1d or 2d; get {rhs_aval.ndim}d.")
+
+  return [dot_general_fn(lhs_data, lhs_indices, lhs_indptr, rhs,
+                         shape=lhs_spinfo.shape, transpose=False,
+                         data_dtype=lhs_data_aval.dtype,
+                         index_dtype=lhs_indices_aval.dtype,
+                         **{x_dtype: rhs_aval.dtype})]
+
 _bcsr_dot_general_default_lowering = mlir.lower_fun(
     _bcsr_dot_general_impl, multiple_results=False)
+
 mlir.register_lowering(
     bcsr_dot_general_p, _bcsr_dot_general_default_lowering)
 
+if gpu_sparse.cuda_is_supported:
+  mlir.register_lowering(bcsr_dot_general_p,
+                          partial(_bcsr_dot_general_gpu_lowering,
+                                  gpu_sparse.cuda_csr_matvec,
+                                  gpu_sparse.cuda_csr_matmat),
+                          platform='cuda')
+if gpu_sparse.rocm_is_supported:
+  mlir.register_lowering(bcsr_dot_general_p,
+                          partial(_bcsr_dot_general_gpu_lowering,
+                                  gpu_sparse.rocm_csr_matvec,
+                                  gpu_sparse.rocm_csr_matmat),
+                          platform='rocm')
+
+
 #----------------------------------------------------------------------
 # BCOO functions that maybe should be primitives?
 
+def bcsr_broadcast_in_dim(mat: BCSR, *, shape: Shape, broadcast_dimensions: Sequence[int]) -> BCSR:
+  result_bcoo = bcoo.bcoo_broadcast_in_dim(
+    mat.to_bcoo(), shape=shape, broadcast_dimensions=broadcast_dimensions)
+  return BCSR.from_bcoo(result_bcoo)
+
+def bcsr_concatenate(operands: Sequence[BCSR], *, dimension: int) -> BCSR:
+  """Sparse implementation of :func:`jax.lax.concatenate`
+
+  Args:
+    operands : Sequence of BCSR arrays to concatenate. The arrays must have equal
+      shapes, except in the `dimension` axis. Additionally, the arrays must have
+      have equivalent batch, sparse, and dense dimensions.
+    dimension : Positive integer specifying the dimension along which to concatenate
+      the arrays. The dimension must be among batch or sparse dimensions of the input;
+      concatenation along dense dimensions is not supported.
+
+  Returns:
+    A BCSR array containing the concatenation of the inputs.
+  """
+  return BCSR.from_bcoo(
+    bcoo.bcoo_concatenate([mat.to_bcoo() for mat in operands], dimension=dimension))
+
 @tree_util.register_pytree_node_class
 class BCSR(JAXSparse):
   """Experimental batched CSR matrix implemented in JAX."""
 
-  data: jnp.ndarray
-  indices: jnp.ndarray
-  indptr: jnp.ndarray
+  data: jax.Array
+  indices: jax.Array
+  indptr: jax.Array
   shape: Shape
   nse = property(lambda self: self.indices.shape[-1])
   dtype = property(lambda self: self.data.dtype)
