@@ -22,12 +22,13 @@ import warnings
 
 import numpy as np
 
-from jax import core
+import jax
 from jax import lax
 from jax.interpreters import mlir
 from jax.experimental.sparse._base import JAXSparse
 from jax.experimental.sparse.util import _coo_extract, CuSparseEfficiencyWarning
 from jax import tree_util
+from jax._src import core
 from jax._src.interpreters import ad
 from jax._src.lax.lax import _const
 from jax._src.lib.mlir.dialects import hlo
@@ -54,9 +55,9 @@ class COO(JAXSparse):
   grad and autodiff, and offers very little functionality. In general you
   should prefer :class:`jax.experimental.sparse.BCOO`.
   """
-  data: jnp.ndarray
-  row: jnp.ndarray
-  col: jnp.ndarray
+  data: jax.Array
+  row: jax.Array
+  col: jax.Array
   shape: Tuple[int, int]
   nse = property(lambda self: self.data.size)
   dtype = property(lambda self: self.data.dtype)
