@@ -21,13 +21,14 @@ import warnings
 
 import numpy as np
 
-from jax import core
+import jax
 from jax.interpreters import mlir
 from jax.experimental.sparse._base import JAXSparse
 from jax.experimental.sparse.coo import _coo_matmat, _coo_matvec, _coo_todense, COOInfo
 from jax.experimental.sparse.util import _csr_to_coo, _csr_extract, CuSparseEfficiencyWarning
 from jax import lax
 from jax import tree_util
+from jax._src import core
 from jax._src.interpreters import ad
 from jax._src.lax.lax import _const
 from jax._src.lib import gpu_sparse
@@ -43,9 +44,9 @@ class CSR(JAXSparse):
   grad and autodiff, and offers very little functionality. In general you
   should prefer :class:`jax.experimental.sparse.BCOO`.
   """
-  data: jnp.ndarray
-  indices: jnp.ndarray
-  indptr: jnp.ndarray
+  data: jax.Array
+  indices: jax.Array
+  indptr: jax.Array
   shape: Tuple[int, int]
   nse = property(lambda self: self.data.size)
   dtype = property(lambda self: self.data.dtype)
@@ -129,9 +130,9 @@ class CSR(JAXSparse):
 @tree_util.register_pytree_node_class
 class CSC(JAXSparse):
   """Experimental CSC matrix implemented in JAX; API subject to change."""
-  data: jnp.ndarray
-  indices: jnp.ndarray
-  indptr: jnp.ndarray
+  data: jax.Array
+  indices: jax.Array
+  indptr: jax.Array
   shape: Tuple[int, int]
   nse = property(lambda self: self.data.size)
   dtype = property(lambda self: self.data.dtype)
