@@ -12,7 +12,12 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+# Note: import <name> as <name> is required for names to be exported.
+# See PEP 484 & https://github.com/google/jax/issues/7570
+
 from jax._src.scipy.stats.bernoulli import (
   logpmf as logpmf,
   pmf as pmf,
+  cdf as cdf,
+  ppf as ppf
 )
